#!/bin/bash -x

# Set default umask
umask 022

# Skip promotion if this file exists
HOLIDAY_FILE=/srv/jenkins/HOLIDAY

# Write the build properties file
cat << EOF > "${WORKSPACE}/build_properties"
SUITE=${SUITE}
STREAM=${STREAM}
SERIAL=${SERIAL}
BUILD_TYPE=${BUILD_TYPE}

EOF

# Write the environmental variables to the run file
env > ${SUITE}.run

fail() { echo "$@"; exit 1;}
dist_ge() { [[ "$1" > "$2" || "$1" == "$2" ]]; }

arches=(i386 amd64 armel armhf arm64 ppc64el)
exec_c="/srv/builder/vmbuilder/bin/cronrun"
rel_base="/srv/ec2-images/releases/${SUITE}/release"
rel_link=$(readlink ${rel_base})

[ "${BUILD_TYPE}" = "desktop" ] &&
    echo "Not valid for desktop builds" &&
    exit 0

# Find the existing manifest file
old_manifest=$(find -L ${rel_base} -maxdepth 1 -iname '*amd64.manifest') ||
    echo "Unable to find release manifest file"

# Find the new manifest file
new_manifest_d="/srv/ec2-images/${SUITE}/${SERIAL}"
[ "${TEST_BUILD:-0}" -eq 1 ] && new_manifest_d="/srv/ec2-images/test_builds/${SUITE}/${SERIAL}"
[ "${SANDBOX_BUILD:-0}" -eq 1 ] && new_manifest_d="/srv/ec2-images/sandbox/${SUITE}/${SERIAL}"
new_manifest=$(find ${new_manifest_d} -maxdepth 1 -iname '*amd64.manifest') ||
    fail "Unable to find new manifest file"

# Find the previous serial if there was one
previous_serial=$(find  /srv/ec2-images/${SUITE}/ -maxdepth 1 -type d |\
    awk -F\/ '{print$NF}' | sort -rn | grep "." | grep -v "${SERIAL}" | head -n1) ||
    echo "Unable to find prior daily manifest"

previous_manifest=${new_manifest//$SERIAL/$previous_serial}

# Generate the pure package diffs
for arch in "${arches[@]}"
do
    nm=${new_manifest//amd64/$arch}
    om=${old_manifest//amd64/$arch}
    pm=${previous_manifest/amd64/$arch}

    [ -e "${nm}" ] &&
        cp "${nm}" "${WORKSPACE}/manifest-${arch}-daily-${SERIAL}.txt"

    # Generate the diff from daily to release
    if [ -e "${nm}" -a -e "${om}" ]; then
        release_diff=${new_manifest##*/}
        release_diff=${release_diff//.manifest/-$rel_link-to-daily_manifest.diff}
        release_diff=${release_diff//amd64/$arch}
        diff -u ${om} ${nm} > "${WORKSPACE}/${release_diff}"
        cp ${om} "${WORKSPACE}/manifest-${arch}-release.txt"
    fi

    # Generate the diff from daily to old daily
    if [ -e "${nm}" -a -e "${pm}" ]; then
        daily_diff=${new_manifest##*/}
        daily_diff=${daily_diff//.manifest/-$previous_serial-to-$SERIAL-manifest.diff}
        daily_diff=${daily_diff//amd64/$arch}
        diff -u ${pm} ${nm} > "${WORKSPACE}/${daily_diff}"
        cp ${pm} "${WORKSPACE}/manifest-${arch}-previous_daily-${previous_serial}.txt"
    fi
done

# Determine if there is a different version of a particular package.
# If so, write out the package information to the specified trigger file
package_differences() {
    # If a pattern is used, it should match a single entry in the manifest
    package_pattern=$1
    trigger_file=$2

    echo "Checking for differences in ${package_pattern}"

    v1="$(awk "/${package_pattern}/ {print\$NF}" ${old_manifest})"
    v2="$(awk "/${package_pattern}/ {print\$NF}" ${new_manifest})"

    if [ "x${v1}" != "x${v2}" ]; then
        echo "  Package changed old:${v1}, new:${v2}"
        cat << PKGDIFF >> ${trigger_file}
'${package_pattern}':
- old: '${v1}'
- new: '${v2}'

PKGDIFF
    else
        echo "  No difference   old:${v1}, new:${v2}"
    fi
}

# Set packages to trigger an automated promotion in this array
# This list of packages is controlled through application of
# lp:~cloudware/cpc-core/+git/cpc_policy:policies/0003_automated_daily_promotion.rst
# trigger_set MUST NOT be modified without accompanying policy doc change
declare -a trigger_set
# This is an array of package names where the string is any awk-friendly
# pattern supported by the expression in package_differences(), but it must
# only match a single package
#trigger_set=('example_package' 'example_package2-*')
trigger_set=('pollinate')

# Append the kernel package to the trigger_set array
if dist_ge ${SUITE} quantal; then
    trigger_set[${#trigger_set[@]}]='linux-image.*generic'
else
    trigger_set[${#trigger_set[@]}]='linux-image-virtual'
fi

# For legacy reasons the jenkins jobs use a "kernel" trigger file
# for automated build promotion.  All package changes will
# use this single trigger file until the need arises for more
# granularity.
trigger_file="${WORKSPACE}/${SUITE}-kernel-trigger"
if [ -e "${trigger_file}" ] ; then
    echo "Cleaning up old trigger file in workspace"
    rm --verbose "${trigger_file}"
fi

# Check all packages in the trigger_set array
for pkg in ${trigger_set[@]} ; do
    package_differences "${pkg}" "${trigger_file}"
done

# If the trigger file exists, determine if it should be pushed to the
# build trigger directory for action
if [ -e "${trigger_file}" ] ; then
    if [ ! -e "${rel_base}" ]; then
        echo "${SUITE} not released, not triggering"
        rm --verbose "${trigger_file}"
    elif [ ! -f ${HOLIDAY_FILE} ]; then
        echo "Creating trigger file with contents:"
        cat "${trigger_file}"
        cp --verbose "${trigger_file}" \
           "${TRIGGER_LOCATION:-/srv/builder/triggers/kernel}/${PARENT_BUILDER_ID}.trigger"
    else
        echo "Not creating trigger, ${HOLIDAY_FILE} found"
        if [ -e "${TRIGGER_LOCATION:-/srv/builder/triggers/kernel}/${PARENT_BUILDER_ID}.trigger" ]; then
            echo "Removing existing trigger from old build with same parent ID."
            rm --verbose "${TRIGGER_LOCATION:-/srv/builder/triggers/kernel}/${PARENT_BUILDER_ID}.trigger"
        fi
    fi
else
    echo "No trigger file found"
fi

# Copy the diffs into the current workspace
cp ${WORKSPACE}/*.diff ${new_manifest_d}/unpacked

# Generate the mfdiff between the dailies
[ -e "${previous_manifest}" -a -e "${new_manifest}" ] &&
    ${exec_c} mfdiff amd64 ${SUITE} ${previous_manifest} ${new_manifest} >\
    "${WORKSPACE}/${SUITE}-daily.changelog"

# Generate the diff between daily and the released image
[ -e "${old_manifest}" -a -e "${new_manifest}" ] &&
    ${exec_c} mfdiff amd64 ${SUITE} ${old_manifest} ${new_manifest} >\
    "${WORKSPACE}/${SUITE}-${rel_link}-to-daily.changelog"

# Copy the changelogs into the current workspace
cp ${WORKSPACE}/*.changelog ${new_manifest_d}/unpacked

# The rest of the operations are for released images only
[ ! -e "${rel_base}" ] &&
    echo "No current release, aborting comparison" &&
    exit 0

# Tar up the deltas
tar -C ${WORKSPACE} -jcvf "${WORKSPACE}/${SUITE}-${SERIAL}.tar.bz2" \
    *.changelog \
    *.txt \
    *.diff ||
    fail "Failed to create tarball"

# Start the email report work
changed_pkgs=$(grep '=>' ${SUITE}-${rel_link}-to-daily.changelog | \
    sed -e 's,====,,g' -e 's,^, *,g' | sort -k2)

# Generate the email template
VER=$(${kvm}/ubuntu-adj2version ${SUITE})

case ${VER} in
    *8.04*)  VER="${VER} LTS";
             CODENAME="Hardy Heron";;
    *10.04*) VER="${VER} LTS";
             CODENAME="Lucid Lynx";;
    *11.04*) CODENAME="Natty Narwhal";;
    *11.10*) CODENAME="Oneiric Ocelot";;
    *12.04*) VER="${VER} LTS";
             CODENAME="Precise Pangolin";;
    *12.10*) CODENAME="Quantal Queztal";;
    *13.04*) CODENAME="Raring Ringtail";;
    *13.10*) CODENAME="Saucy Salamander";;
    *14.04*) VER="${VER} LTS";
             CODENAME="Trusty Tahr";;
    *14.10*) CODENAME="Utopic Unicorn";;
    *15.04*) CODENAME="Vivid Vervet";;
    *15.10*) CODENAME="Wily Werewolf";;
esac

email_name="${WORKSPACE}/${SUITE}-release_announcement.email"
cat << EOF > "${email_name}"
SUBJECT: Refreshed Cloud Images of ${VER} (${CODENAME}) [${SERIAL}]
TO: ec2ubuntu@googlegroups.com; ubuntu-cloud@lists.ubuntu.com; ubuntu-cloud-announce@lists.ubuntu.com

A new release of the Ubuntu Cloud Images for stable Ubuntu release ${VER} (${CODENAME}) is available at [1]. These new images superseded the existing images [2]. Images are available for download or immediate use on EC2 via publish AMI ids. Users who wish to update their existing installations can do so with:
   'sudo apt-get update && sudo apt-get dist-upgrade && sudo reboot'.

EOF

if [ "${old_linux_kernel}" != "${new_linux_kernel}" ]; then
    cat << EOF >> "${email_name}"
The Linux kernel was updated from ${old_linux_kernel} [3] to ${new_linux_kernel} [4]

EOF
fi

cat << EOF >> "${email_name}"
The following packages have been updated. Please see the full changelogs
for a complete listing of changes:
${changed_pkgs}


The following is a complete changelog for this image.
$(cat ${SUITE}-${rel_link}-to-daily.changelog)

--
[1] http://cloud-images.ubuntu.com/releases/${SUITE}/release-${SERIAL}/
[2] http://cloud-images.ubuntu.com/releases/${SUITE}/${rel_link}/
EOF

if [ "${old_linux_kernel}" != "${new_linux_kernel}" ]; then
cat << EOF >> "${email_name}"
[3] http://changelogs.ubuntu.com/changelogs/pool/main/l/linux/linux_${old_linux_kernel}/changelog
[4] http://changelogs.ubuntu.com/changelogs/pool/main/l/linux/linux_${new_linux_kernel}/changelog
EOF
fi

# Create release notes
lnc=$(wc -l ${email_name} | awk '{print$1}')
tail -n `expr $lnc - 3` ${email_name} > "${WORKSPACE}/release_notes.txt"
cp ${WORKSPACE}/release_notes.txt ${new_manifest_d}/unpacked
