#!/bin/bash
retcode=0
error() { echo "$@" 1>&2; }
fail() { error "$@"; sleep 120; retcode=1; exit 1; }
debug() { error "$(date -R):" "$@"; }
#Look for REGISTRATION text. This contains the information

export PATH="${PATH}:/usr/local/bin:/usr/bin:${WORKSPACE}"
export TMPDIR="${WORKSPACE}/tmp"
TMP=${TMPDIR}
mkdir -p ${TMPDIR}

#about what to test. This is important.
REGISTRATION_RECORD="${WORKSPACE}/build_properties"
[ -z "${OVERRIDE}" ] && {
    [ -e "${REGISTRATION_RECORD}" ] && {
    source ${REGISTRATION_RECORD} ||
        fail "Registration information is missing. Failing."
    }
}

#Setup control variables
test_suite=${1:-$TEST}
def_size=${VMSIZE:-Small}
def_user="ubutest"
def_pass="$(openssl rand -base64 13 | awk '-F=' '{print$1}')#1"
vm_name_base="test-$(date +%Y%m%d)-${BUILD_NUMBER:-NA}-$(openssl rand -hex 4)"
running=()  # collection of images running

#Reset some pathing
export HOME="${WORKSPACE}"
export PATH="${NODE_LIB}:${LOCAL_LIB}:${LOCAL_LOCAL}:${LOCAL_BIN}:${PATH}"
my_dir="$( cd "$( dirname "$0" )" && pwd )"
base_dir=$(dirname ${my_dir})

#Sanity checking!
[ -z "${SUITE}" ] && fail "Suite is undefined!"
[ -z "${SERIAL}" ] && fail "Serial is undefined!"
[ -z "${REGISTRATION}" ] && fail "Registration name is undefined"

# The certificate should exist in the workspace, base64 encoded
# The file should be at ${WORKSPACE}/cred.b64

cleaner() {
    [ -e ${WORKSPACE}/.azure ] && wipe -R ${WORKSPACE}/.azure
    }

trap cleaner EXIT

export HOME=${WORKSPACE}
export azure="/usr/local/bin/azure"
[ -x ${azure} ] || azure=$(which azure) ||
    fail "unable to find executable for azure cli command"

# Unpack the security creds
base64 -d ${WORKSPACE}/cred.b64 > ${WORKSPACE}/creds
${azure} account import ${WORKSPACE}/creds ||
         fail "Failed to decode the credential file"

${azure} vm list

#Output something pretty
set +x
cat <<EOM
_________________________________________________________________

*****************************************************************

TESTING:
    Suite: ${SUITE}
    Serial: ${SERIAL}
    Registration: ${REGISTRATION}

Test: ${test_suite}

*****************************************************************
_________________________________________________________________
EOM
set -x

#Retry command
retry() {
    local trycount=${1} sleep=${2}
    shift; shift;
    local i=0 smsg=" sleeping ${sleep}: $*" ret=0
    for((i=0;i<${trycount};i++)); do
        "$@" && return 0
        ret=$?
        [ $(($i+1)) -eq ${trycount} ] && smsg=""
        debug 1 "Warning: cmd failed [try $(($i+1))/${trycount}].${smsg}"
        sleep $sleep
    done
    return $ret
}

#Kill instance
terminate() {
    retry 3 30 azure vm delete -b -q ${1} ||
        error "Failed to delete VM ${1}"

    #remove it from list of running images
    running=(${running[@]//${1}/})
}

#Kill all instances
terminate_all() {
    for img in ${running[@]}
    do
        terminate ${img}
    done
}

#Make sure any exit, kills all running instances
trap terminate_all EXIT

#Launch an instance
run_instance() {
    local vmname=${1}
    local extras=${@};
    [ "${extras[@]}" -eq 1 ] && extras=${extra[@]:1}

    custom_data=$(mktemp)
    cat >> ${custom_data} <<END
#cloud-config
output: {all: '| tee -a /var/log/cloud-init-output.log'}
ssh_import_id: [utlemming,rcj,daniel-thewatkins]
packages:
- bzr
- git
- pastebinit
runcmd:
- [ echo, "${BUILD_TAG} Testing TAG" ]
- [ touch, /tmp/cloud-init.done ]
END

    local run_cmd=(
           ${azure} vm create
           ${vmname}
           ${REGISTRATION}
           --location 'North Europe'
           --ssh 22
           --userName ${def_user}
           --password ${def_pass}
           --vm-size ${def_size}
           --custom-data ${custom_data}
           ${extras[@]}
        )

    run_cmd+=(--verbose --json)

    debug 1 "CMD: ${run_cmd[@]}"
    retry 3 600 "${run_cmd[@]}" || fail "Unable to launch instance"
    running+=(${1})
}

check_instance() {
    local status=$(azure vm show ${1} | awk '-F"' '/InstanceStatus/ {print$2}')
    case ${status} in
        ReadyRole)          ssh-keyscan -T 60 -t rsa ${1}.cloudapp.net && return 0;
                            debug 1 "instance is not quite ready yet..."; return 1;;
        FailedProvisioning) fail "Failed to provision VM";;
        FailedStartingVM)   fail "Failed to start VM";;
        Failed*)            fail "General error in VM, status ${status}";;
        *)                  debug "Status is ${status}"; return 1;;
    esac
    return 1
}

get_dns() {
    dns=$(${azure} vm show ${1} |
                 awk '-F"' '/DNSName/  {print$2}')

    [ -n "${dns}" ] && echo "${dns}" ||
        fail "Unable to determine DNS name for ${1}"
}

wait_for() {
    local sleeptime=${1}
    local max_count=${2}
    local command=(${@})
    unset command[0]; unset command[1];
    local count=0

    while $(sleep ${sleeptime}); do
        ${command[@]} && return 0
        count=$(expr ${count} + 1 )

        if [ "${count}" -ge "${max_count}" ]; then
            return 1
        fi
    done
}


# Test password-based SSH
password_ssh() {
    which sshpass 2> /dev/null &>1 ||
        sudo apt-get -y install sshpass ||
            fail "Required program, sshpass, is not installed"

    # create instance
    local vmname="${vm_name_base}-pass"
    run_instance ${vmname}

    wait_for 30 60 check_instance ${vmname} ||
        fail "Instance failed to initialize in time"

    local dnsname=$(get_dns ${vmname})
    error "Sleeping for 3 mintues...to let things settle"
    sleep 180

    # Give ourselves sudo ability.....
    # This is pretty much brute force...but it does test sudo
    error "Clobbering sudo permissions via brute force..."
    ${my_dir}/passless-sudoifer --host ${dnsname} --user ${def_user} \
                                --passwd ${def_pass}

    # Run Jenkins test runner
    error "Running test suite..."
    ${my_dir}/jenkins-ssh --host ${dnsname} --user ${def_user} \
                          --passwd ${def_pass} --file ${my_dir}/run-azure.sh &&
        error "Test suite passed successfully" ||
        fail  "Test suite failed run"

}

# Test passwordless ssh
key_ssh() {
    # SSH key creation for Azure posted here:
    # https://www.windowsazure.com/en-us/manage/linux/how-to-guides/ssh-into-linux/
    openssl req \
        -x509 \
        -batch \
        -nodes \
        -days 365 \
        -newkey rsa:2048 \
        -keyout myPrivateKey.key \
        -out myCert.pem
    chmod 600 myPrivateKey.key
    openssl  x509 -outform der -in myCert.pem -out cert.pem ||
        fail "Failed to generate SSH certificate"

    ssh-keygen -e -f myPrivateKey.key > openssh.pem ||
        fail "Unable to create public SSH certificate"

    openssl rsa -in myPrivateKey.key -out ${WORKSPACE}/PrivateKey.key

    # Launch the instance
    vmname="${vm_name_base}-key"
    vmargs=(${vmname}
            --ssh-cert ${WORKSPACE}/myCert.pem
            --no-ssh-password)
    run_instance ${vmargs[@]}

    wait_for 15 60 check_instance ${vmname} ||
        fail "Instance failed to initialize in time"

    dnsname=$(get_dns ${vmname})

    # Run Jenkins test runner
    error "Running test suite..."
    retry 2 180 \
        ${my_dir}/jenkins-ssh --host ${dnsname} \
                          --user ${def_user} \
                          --key ${WORKSPACE}/PrivateKey.key \
                          --file ${my_dir}/run-azure.sh &&
        error "Test suite passed successfully" ||
        fail  "Test suite failed run"
   }

# Look at image characteristics
chacteristics() {
    exit 0
}

case ${TEST} in
    Password-SSH*)       password_ssh;;
    Key-SSH*)            key_ssh;;
    Characteristics*)    chacteristics;;
esac

terminate_all

# Unset trap
trap - EXIT
exit ${retcode}
